#! /usr/bin/bash

RAW_BASE_IMAGE="ubuntu-20.04.1.img"
RUNNING_IMAGE="ubuntu_running.qcow2"
INSTALL_IMAGE="ubuntu-22.04.2-live-server-amd64.iso"

function pause() {
    echo -n "Press Enter to continue..."
    read
    echo
}

echo "Create blank disk image, 10GB"
echo "-------------------------------------------------------------------------"
echo "$ qemu-img create -f raw \"$RAW_BASE_IMAGE\" 10G"

qemu-img create -f raw "$RAW_BASE_IMAGE" 10G \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "Run VM for installation: run on the disk image, booting from the installation media"
echo -e "\t-enable-kvm makes it do hardware virtualization (instead of emulation)"
echo -e "\t-cpu host is used to enable all HW-assisted virtualization features"
echo -e "\talso give 8GB of memory (default is 128MB) and 4vCPU"
echo "-------------------------------------------------------------------------"
echo -e "$ qemu-system-x86_64 -name \"install_ubuntu\" \n\
    \t-enable-kvm -cpu host \n\
    \t-cdrom $INSTALL_IMAGE -boot order=d \n\
    \t-drive file=\"$RAW_BASE_IMAGE\",format=raw \n\
    \t-m 8G -smp cpus=4"

qemu-system-x86_64 -name "install_ubuntu" \
    -enable-kvm -cpu host \
    -cdrom "$HOME/$INSTALL_IMAGE" -boot order=d \
    -drive file="$RAW_BASE_IMAGE",format=raw \
    -m 8G -smp cpus=4 \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

sudo ip link add name qemubr0 type bridge 2> /dev/null && sudo ip link set dev qemubr0 up

echo "Run VM: run on the disk image, booting from it"
echo -e "\tsame options, completed with video and network configuration"
echo -e "\t-display sets the display mode, -device #1 configures the virtual graphic device"
echo -e "\t-netdev sets the network mode, -device #2 configures the virtual network device"
echo -e "\tmentions of virtio mean usage of paravirtualized drivers"
echo "-------------------------------------------------------------------------"
echo -e "$ qemu-system-x86_64 -name \"demo_ubuntu\" \n\
    \t-enable-kvm -cpu host \n\
    \t-display gtk,gl=on \n\
    \t-drive file=\"$RUNNING_IMAGE\",format=qcow2,if=virtio \n\
    \t-m 8G -smp cpus=4 \n\
    \t-device virtio-gpu-gl \n\
    \t-netdev bridge,br=qemubr0,id=br0 \n\
    \t-device virtio-net,netdev=br0"

qemu-system-x86_64 -name "demo_ubuntu" \
    -enable-kvm -cpu host \
    -display gtk,gl=on \
    -drive file="$RUNNING_IMAGE",format=qcow2,if=virtio \
    -m 8G -smp cpus=4 \
    -device virtio-gpu-gl \
    -netdev bridge,br=qemubr0,id=br0 \
    -device virtio-net,netdev=br0 \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "Run VM with less resources: 2GB, 2vCPU, no network, no paravirtualization, no HW-assisted virtualization"
echo "-------------------------------------------------------------------------"
echo -e "$ qemu-system-x86_64 -name \"demo_ubuntu_light\" \n\
    \t-enable-kvm \n\
    \t-display gtk,gl=on \n\
    \t-drive file=\"$RUNNING_IMAGE\",format=qcow2 \n\
    \t-m 2G -smp cpus=2 \n\
    \t-nic none"

qemu-system-x86_64 -name "demo_ubuntu_light" \
    -enable-kvm \
    -display gtk,gl=on \
    -drive file="$RUNNING_IMAGE",format=qcow2 \
    -m 2G -smp cpus=2 \
    -nic none \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

